@@ -1,6 +1,5 @@
 fitted.arx <-
-function(object, spec=NULL,
-  ...)
+function(object, spec=NULL, ...)
 {
   ##spec argument:
   if(is.null(spec)){
